# import db

# import pandas as pd
# import numpy as np
# from tabulate import tabulate
# from datetime import datetime as dt

# from create_table import create_table
# from filter_table import filter
# from select_from_table import select

import db_mimic


def get_table(tablename):
    return meta_data[tablename]


def show(table, rows):
    if not isinstance(table, pd.DataFrame):
        print("\nSHOW ERROR: No table to show.")
        return
    print(f"\nShowing {rows} rows from table.")
    if len(table) < rows:
        print(f"This table contains fewer than {rows} rows.")
    print(
        tabulate(
            table.head(rows),
            headers="keys",
            tablefmt="fancy_grid",
            showindex="always",
        )
    )
    return table


def is_float(val):
    """
    method to determine if string is a float
    input: string
    computation: try to cast to float
    output: boolean (True = string can be cast to float)
    """
    try:
        float(val)
    except:
        return False
    return True


def is_int(val):
    """
    method to determine if string is a int
    input: string
    computation: try to cast to int
    output: boolean (True = string can be cast to int)
    """
    try:
        int(val)
    except:
        return False
    return True



import db

# ? select(tablename, columns) -> select values under specified columns from specified table
# * advanced: accept expressions over columns
def select(tablename, columns):
    """
    select values from specified columns
    input: tablename, columns as list of table indices
    computation:
        pull table based on tablename
        select specified columns
        copy into temp table
    output: temp table of selected columns
    """
    if len(columns) < 1:
        print(f"\nSELECT ERROR: Must select at least one column from {tablename}.")
        return

    # determine data types of columns passed in; retrieve accordingly
    column_inputs = [type(x) == str for x in columns]
    if False in column_inputs and True in column_inputs:
        print(
            "\nSELECT ERROR: Invalid selection. Please enter column names OR column indices."
        )
        return

    table = db.get_table(tablename)[1]  #! if switch to dict, [1] ==> ['table']

    if False in column_inputs:  # columns = list of indices
        temp_table = table.iloc[:, columns]
    else:  # columns = list of headers
        temp_table = table.loc[:, columns]

    print(f"Retrieved columns {columns} from {tablename}.")
    return temp_table



import db
import pandas as pd
from datetime import datetime as dt


def create_table(tablename, filename, has_headers=True):
    """
    creates table, tablename, from file, filename
    file MUST be .csv

    input: desired name of table, filename to be imported
        optional: has_headers default is True & first row of file will be treated as headers for table if set to False, first row of file will be the first row of data in the table
    computation:
        extract headers from .csv
        loop through all row of file, creating new table entry for each row
    output: table
    """
    # require .csv file
    filetype = filename[-3:]
    if filetype != "csv":
        print("not a valid filetype, must pass in a .csv")
        return

    # create meta_data
    # created_at =
    # open and read in file
    if has_headers == True:
        table = pd.read_csv(filename, header=0)
    else:
        table = pd.read_csv(filename, header=None)
    # add table to meta_data
    # todo: change to dictionary to access from key instead of index?
    # todo ==> meta_data[tablename] = {has_headers: boolean, table: pd.df, created_at: dt.now()}
    db.meta_data[tablename] = [has_headers, table, dt.now()]

    # confirm successful table creation & addition to meta_data
    print(f"table: '{tablename}' successfully created.")
    return table


if __name__ == "__main__":
    create_table(tablename, has_headers)


import db

# ? filter(tablename, condition) -> select all entries from specified table where condition == True
# * advanced: accept multiple conditions
def filter(tablename, condition):
    table = db.get_table(tablename)[1]
    print(f"\n\nFiltering '{tablename}' for col {condition}")
    if ">=" in condition:
        cond = condition.split(">=")
        col = cond[0].strip()
        val = cond[-1]
        if db.is_int(col):  # col is int
            col = int(col)
            if db.is_float(val):
                val = float(val)
                temp_table = table[table.iloc[:, col] >= val]  # val is float
                return temp_table
            elif db.is_int(val):
                val = int(val)
                temp_table = table[table.iloc[:, col] >= val]  # val is int
                return temp_table
            else:
                temp_table = table[table.iloc[:, col] >= val.strip()]  # val is str
                return temp_table
        else:  # col is str
            if db.is_float(val):
                val = float(val)
                temp_table = table[table.loc[:, col] >= val]  # val is float
                return temp_table
            elif db.is_int(val):
                val = int(val)
                temp_table = table[table.loc[:, col] >= val]  # val is int
                return temp_table
            else:
                temp_table = table[table.loc[:, col] >= val.strip()]  # val is str
                return temp_table

    elif ">" in condition:
        cond = condition.split(">")
        col = cond[0].strip()
        val = cond[-1]
        if db.is_int(col):  # col is int
            col = int(col)
            if db.is_float(val):
                val = float(val)
                temp_table = table[table.iloc[:, col] > val]  # val is float
                return temp_table
            elif db.is_int(val):
                val = int(val)
                temp_table = table[table.iloc[:, col] > val]  # val is int
                return temp_table
            else:
                temp_table = table[table.iloc[:, col] > val.strip()]  # val is str
                return temp_table
        else:  # col is str
            if db.is_float(val):
                val = float(val)
                temp_table = table[table.loc[:, col] > val]  # val is float
                return temp_table
            elif db.is_int(val):
                val = int(val)
                temp_table = table[table.loc[:, col] > val]  # val is int
                return temp_table
            else:
                temp_table = table[table.loc[:, col] > val.strip()]  # val is str
                return temp_table

    elif "<=" in condition:
        cond = condition.split("<=")
        col = cond[0].strip()
        val = cond[-1]
        if db.is_int(col):  # col is int
            col = int(col)
            if db.is_float(val):
                val = float(val)
                temp_table = table[table.iloc[:, col] <= val]  # val is float
                return temp_table
            elif db.is_int(val):
                val = int(val)
                temp_table = table[table.iloc[:, col] <= val]  # val is int
                return temp_table
            else:
                temp_table = table[table.iloc[:, col] <= val.strip()]  # val is str
                return temp_table
        else:  # col is str
            if db.is_float(val):
                val = float(val)
                temp_table = table[table.loc[:, col] <= val]  # val is float
                return temp_table
            elif db.is_int(val):
                val = int(val)
                temp_table = table[table.loc[:, col] <= val]  # val is int
                return temp_table
            else:
                temp_table = table[table.loc[:, col] <= val.strip()]  # val is str
                return temp_table

    elif "<" in condition:
        cond = condition.split("<")
        col = cond[0].strip()
        val = cond[-1]
        if db.is_int(col):  # col is int
            col = int(col)
            if db.is_float(val):
                val = float(val)
                temp_table = table[table.iloc[:, col] < val]  # val is float
                return temp_table
            elif db.is_int(val):
                val = int(val)
                temp_table = table[table.iloc[:, col] < val]  # val is int
                return temp_table
            else:
                temp_table = table[table.iloc[:, col] < val.strip()]  # val is str
                return temp_table
        else:  # col is str
            if db.is_float(val):
                val = float(val)
                temp_table = table[table.loc[:, col] < val]  # val is float
                return temp_table
            elif db.is_int(val):
                val = int(val)
                temp_table = table[table.loc[:, col] < val]  # val is int
                return temp_table
            else:
                temp_table = table[table.loc[:, col] < val.strip()]  # val is str
                return temp_table
    elif "==" in condition:
        cond = condition.split("==")
        col = cond[0].strip()
        val = cond[-1]
        if db.is_int(col):  # col is int
            col = int(col)
            if db.is_float(val):
                val = float(val)
                temp_table = table[table.iloc[:, col] == val]  # val is float
                return temp_table
            elif db.is_int(val):
                val = int(val)
                temp_table = table[table.iloc[:, col] == val]  # val is int
                return temp_table
            else:
                temp_table = table[table.iloc[:, col] == val.strip()]  # val is str
                return temp_table
        else:  # col is str
            if db.is_float(val):
                val = float(val)
                temp_table = table[table.loc[:, col] == val]  # val is float
                return temp_table
            elif db.is_int(val):
                val = int(val)
                temp_table = table[table.loc[:, col] == val]  # val is int
                return temp_table
            else:
                temp_table = table[table.loc[:, col] == val.strip()]  # val is str
                return temp_table
    elif "=" in condition:
        cond = condition.split("=")
        col = cond[0].strip()
        val = cond[-1]
        if db.is_int(col):  # col is int
            col = int(col)
            if db.is_float(val):
                val = float(val)
                temp_table = table[table.iloc[:, col] == val]  # val is float
                return temp_table
            elif db.is_int(val):
                val = int(val)
                temp_table = table[table.iloc[:, col] == val]  # val is int
                return temp_table
            else:
                temp_table = table[table.iloc[:, col] == val.strip()]  # val is str
                return temp_table
        else:  # col is str
            if db.is_float(val):
                val = float(val)
                temp_table = table[table.loc[:, col] == val]  # val is float
                return temp_table
            elif db.is_int(val):
                val = int(val)
                temp_table = table[table.loc[:, col] == val]  # val is int
                return temp_table
            else:
                temp_table = table[table.loc[:, col] == val.strip()]  # val is str
                return temp_table
    else:
        print("Invalid condition argumen. Cannot perform filter.")
        return table



import db

# * create_table test
db.create_table("test", "test_data.csv")
# db.create_table("greenhouse", "greenhouse-gas-emissions.csv")
# db.create_table("err_table", "wrong_type") # returns error for wrong data type

# * select test
db.show(db.select("test", [0, 1, 2]), 5)  # indices
# db.show(db.select("test", ["region", "units", "gas"]), 6)  # col ndb.s
# db.show(db.select("test", ["as", 234, "asdf", 456, 34]), 5)  # ndb.s & indices
# db.show(db.select("test", []), 4)  # no cols selected
# db.show(
#     db.select("greenhouse", [0, 1, 2, 3, 4, 5, 6]), len(db.get_table("greenhouse")[1])
# )
# db.show(db.select("greenhouse", [0, 1, 2, 3, 4, 5, 6]), 20)


# db.show(db.get_table("greenhouse")[1], 10)
# db.show(db.get_table("test")[1], 10)
# * filter test
# ? testing col == int; val == int, float, string
# db.show(db.filter("test", "5 > 2009"), 6)  # * works
# db.show(db.filter("test", "5 < 2009"), 6)  # * works
# db.show(db.filter("test", "5 == 2009"), 6)  # * works
# db.show(db.filter("test", "5 = 2009"), 6)  # * works
# db.show(db.filter("test", "5 >= 2009"), 6)  # * works
# db.show(db.filter("test", "5 <= 2009"), 6)  # * works
# db.show(db.filter("greenhouse", "anzsic_descriptor = Total"), 45)

# db.show(db.filter("test", "0 > Gisborne"), 6)  # * works
# db.show(db.filter("test", "0 < Gisborne"), 6)  # * works
# db.show(db.filter("test", "0 == Gisborne"), 6)  # * works
# db.show(db.filter("test", "0 = Gisborne"), 6)  # * works
# db.show(db.filter("test", "0 >= Gisborne"), 6)  # * works
# db.show(db.filter("test", "0 <= Gisborne"), 6)  # * works

# db.show(db.filter("test", "6 > 850.0"), 6)  # * works
# db.show(db.filter("test", "6 < 850.0"), 6)  # * works
# db.show(db.filter("test", "6 == 1471.69"), 6)  # * works
# db.show(db.filter("test", "6 = 1471.69"), 6)  # * works
# db.show(db.filter("test", "6 >= 850.0"), 6)  # * works
# db.show(db.filter("test", "6 <= 850.0"), 6)  # * works

# # ? testing col == str; val == int, float, string
# db.show(db.filter("test", "year > 2009"), 6)  # * works
# db.show(db.filter("test", "year < 2009"), 6)  # * works
# db.show(db.filter("test", "year == 2009"), 6)  # * works
# db.show(db.filter("test", "year = 2009"), 6)  # * works
# db.show(db.filter("test", "year >= 2009"), 6)  # * works
# db.show(db.filter("test", "year <= 2009"), 6)  # * works

# db.show(db.filter("test", "data_val > 1605.5"), 6)  # * works
# db.show(db.filter("test", "data_val < 1605.5"), 6)  # * works
# db.show(db.filter("test", "data_val == 811.63"), 6)
# db.show(db.filter("test", "data_val = 811.63"), 6)
# db.show(db.filter("test", "data_val >= 1605.5"), 6)  # * works
# db.show(db.filter("test", "data_val <= 1605.5"), 6)  # * works

# db.show(db.filter("test", "region > Gisborne"), 6)  # * works
# db.show(db.filter("test", "region < Gisborne"), 6)  # * works
# db.show(db.filter("test", "region == Gisborne"), 6)  # * works
# db.show(db.filter("test", "region = Gisborne"), 6)  # * works
# db.show(db.filter("test", "region >= Gisborne"), 6)  # * works
# db.show(db.filter("test", "region <= Gisborne"), 6)  # * works


# * show test
temp_table = db.filter("test", "6 < 1605.0")
db.show(temp_table, 8)

# * additional
# print(db.select(db.filter("test", "year > 2009"), [0, 2, 5])) #! not chainable because .filter returns DataFrdb. & .select accepts just tablendb.


from setuptools import setup, find_packages

VERSION = "0.0.3"
DESCRIPTION = "package to emulate simple db functions"
LONG_DESCRIPTION = "package to emulate simple db functions: create_table, select_from_table, show_table, filter_table"

# Setting up
setup(
    name="db_mimic",
    version=VERSION,
    author="Alex Kuhn",
    author_email="<alex.kuhn1123@gmail.com>",
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=[],
    keywords=["python", "db_mimic"],
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Education",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 3",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ],
)